import io

import flask_admin
import logging
from flask import Flask
from flask import request
from flask_admin.contrib.peewee import ModelView

from eyewitness.image_id import ImageId
from eyewitness.models.db_proxy import DATABASE_PROXY
from eyewitness.models.detection_models import (ImageInfo, BboxDetectionResult)
from eyewitness.image_utils import ImageHandler


class BboxObjectDetectionFlaskWrapper(object):
    def __init__(self, obj_detector, image_register, detection_result_handlers,
                 database, with_admin=False, drawn_image_dir=None):
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)
        app = Flask(__name__)
        self.app = app
        self.obj_detector = obj_detector
        self.detection_result_handlers = detection_result_handlers
        self.database = database
        self.image_register = image_register
        self.drawn_image_dir = drawn_image_dir
        DATABASE_PROXY.initialize(self.database)
        ImageInfo.create_table()
        BboxDetectionResult.create_table()

        if with_admin:
            admin = flask_admin.Admin(app, name='Example: Peewee')
            admin.add_view(ModelView(ImageInfo))
            admin.add_view(ModelView(BboxDetectionResult))

        @app.route("/detect_post_bytes", methods=['POST'])
        def detect_image_bytes_objs():
            image_id = ImageId.from_str(request.headers['image_id'])
            self.image_register.register_image(image_id, {})
            raw_image_path = request.headers.get('raw_image_path', None)

            # read data from Bytes
            data = request.data
            image_data_raw = io.BytesIO(bytearray(data))
            image_raw = ImageHandler.read_image_bytes(image_data_raw)

            if raw_image_path:
                ImageHandler.save(image_raw, raw_image_path)

            # detect objs
            detection_result = self.obj_detector.detect(image_raw, image_id)

            # draw and save image, as object detected update detection result
            if self.drawn_image_dir and len(detection_result.detected_objects) > 0:
                self.draw_bbox_for_detection_result(image_id, image_raw, detection_result)

            for detection_result_handler in self.detection_result_handlers:
                detection_result_handler.handle(detection_result)
            return "successfully detected"

        @app.route("/detect_post_path", methods=['POST'])
        def detect_image_file_objs():
            image_id = ImageId.from_str(request.headers['image_id'])
            self.image_register.register_image(image_id, {})
            raw_image_path = request.headers['raw_image_path']
            image_raw = ImageHandler.read_image_file(raw_image_path)
            detection_result = self.obj_detector.detect(image_raw, image_id)

            # draw and save image, as object detected update detection result
            if self.drawn_image_dir and len(detection_result.detected_objects) > 0:
                self.draw_bbox_for_detection_result(image_id, image_raw, detection_result)

            for detection_result_handler in self.detection_result_handlers:
                detection_result_handler.handle(detection_result)
            return "successfully detected"

        def draw_bbox_for_detection_result(self, image_id, image_raw, detection_result):
            drawn_image_path = "%s/%s_%s.%s" % (
                self.drawn_image_dir, image_id.channel, image_id.timestamp,
                image_id.file_format)
            ImageHandler.draw_bbox(image_raw, detection_result.detected_objects)
            ImageHandler.save(image_raw, drawn_image_path)
            detection_result.image_dict['drawn_image_path'] = drawn_image_path
